import { BehaviorSubject } from 'rxjs';
import { MatProgressBarModule } from '@angular/material';
import { CommonModule } from '@angular/common';
import { HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { Inject, Injectable, Component, HostBinding, Input, NgModule } from '@angular/core';
import { finalize } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const LOADING_BAR_ITEMS = 'loading_bar_items';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingBarService {
    /**
     * @param {?} items
     */
    constructor(items) {
        this.items = items;
        this.state = this.items.length !== 0;
        this.stateSubject = new BehaviorSubject(this.state);
        this.isLoading = this.stateSubject.asObservable();
    }
    /**
     * @private
     * @return {?}
     */
    next() {
        if ((this.items.length !== 0) !== this.state) {
            this.state = !this.state;
            this.stateSubject.next(this.state);
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    present(item) {
        /** @type {?} */
        const t = setTimeout((/**
         * @return {?}
         */
        () => {
            this.items.push(item);
            this.next();
            clearTimeout(t);
        }));
    }
    /**
     * @param {?} item
     * @return {?}
     */
    complete(item) {
        /** @type {?} */
        const t = setTimeout((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const index = this.items.indexOf(item);
            if (index !== -1) {
                this.items.splice(index, 1);
                this.next();
            }
            clearTimeout(t);
        }));
    }
    /**
     * @return {?}
     */
    dismissAll() {
        /** @type {?} */
        const t = setTimeout((/**
         * @return {?}
         */
        () => {
            this.items = [];
            this.next();
            clearTimeout(t);
        }));
    }
}
LoadingBarService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
LoadingBarService.ctorParameters = () => [
    { type: Array, decorators: [{ type: Inject, args: [LOADING_BAR_ITEMS,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingBarComponent {
    /**
     * @param {?} service
     */
    constructor(service) {
        this.service = service;
        this.mode = 'indeterminate';
        this.color = 'primary';
        this.position = 'fixed';
        this.top = 0;
        this.left = 0;
        this.right = 0;
        this.zIndex = 1001;
    }
}
LoadingBarComponent.decorators = [
    { type: Component, args: [{
                selector: 'mat-loading-bar',
                template: `
    <mat-progress-bar [mode]="mode" [color]="color" [value]="value" *ngIf="service.isLoading|async"></mat-progress-bar>`
            }] }
];
/** @nocollapse */
LoadingBarComponent.ctorParameters = () => [
    { type: LoadingBarService }
];
LoadingBarComponent.propDecorators = {
    mode: [{ type: Input }],
    color: [{ type: Input }],
    value: [{ type: Input }],
    position: [{ type: HostBinding, args: ['style.position',] }, { type: Input }],
    top: [{ type: HostBinding, args: ['style.top',] }, { type: Input }],
    left: [{ type: HostBinding, args: ['style.left',] }, { type: Input }],
    right: [{ type: HostBinding, args: ['style.right',] }, { type: Input }],
    bottom: [{ type: HostBinding, args: ['style.bottom',] }, { type: Input }],
    zIndex: [{ type: HostBinding, args: ['style.z-index',] }, { type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingBarInterceptor {
    /**
     * @param {?} service
     */
    constructor(service) {
        this.service = service;
    }
    /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    intercept(req, next) {
        if (req.headers.has('skipLoadingBar')) {
            return next.handle(req.clone({ headers: req.headers.delete('skipLoadingBar') }));
        }
        /** @type {?} */
        const r = next.handle(req);
        /** @type {?} */
        let started;
        /** @type {?} */
        const responseSubscribe = r.subscribe.bind(r);
        r.subscribe = (/**
         * @param {...?} args
         * @return {?}
         */
        (...args) => {
            started = Math.random().toString(36).substring(7);
            this.service.present(started);
            return responseSubscribe(...args);
        });
        return r.pipe(finalize((/**
         * @return {?}
         */
        () => !!started && this.service.complete(started))));
    }
}
LoadingBarInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
LoadingBarInterceptor.ctorParameters = () => [
    { type: LoadingBarService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = [];
class LoadingBarModule {
}
LoadingBarModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    LoadingBarComponent
                ],
                imports: [
                    CommonModule,
                    HttpClientModule,
                    MatProgressBarModule
                ],
                providers: [
                    { provide: LOADING_BAR_ITEMS, useValue: ɵ0 },
                    { provide: HTTP_INTERCEPTORS, useClass: LoadingBarInterceptor, multi: true },
                    LoadingBarService
                ],
                exports: [
                    LoadingBarComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { LOADING_BAR_ITEMS, LoadingBarService, LoadingBarComponent, LoadingBarModule, LoadingBarInterceptor as ɵa };

//# sourceMappingURL=ngxlib-loading-bar.js.map